#include <QtDebug>
#include <QTime>
#include "MainWnd.h"

MainWnd::MainWnd( QMainWindow * parent, Qt::WindowFlags flags ) : QMainWindow(parent, flags) {
    // Initialization
    cam = cvCreateCameraCapture(0);
    assert(cam);

    gaussSzX = 5; gaussSzY = 5;
    gaussSig1 = 1.5; gaussSig2 = 1.5;
    cannyTr1 = 96.0; cannyTr2 = 128.0;
    activePic = 3;

    // GUI
    // Central-Widget enthlt Kamerabild und eine Sidbar
    central = new QWidget(this);
    layCentral = new QHBoxLayout(central);
    layCentral->setMargin(0);

    // Widget fr das Kamerabild
    imPic = new QImage(320,240,QImage::Format_RGB32);

    for (int x = 0; x < 320; x ++) {
        for (int y = 0; y < 240; y++) {
            imPic->setPixel(x,y,qRgb(0, 0, 0));
        }
    }

    laPic = new QLabel;
    laPic->setPixmap(QPixmap::fromImage(*imPic));
    layCentral->addWidget(laPic);

    // Widget fr die Sidebar
    sidebar = new QWidget(central);
    sidebar->setMinimumWidth(100);
    laySidebar = new QVBoxLayout(sidebar);
    laySidebar->setMargin(5);

    // Widget fr die Auswahl des Kamerabildes
    cbPic = new QComboBox(sidebar);
    cbPic->setEditable(false);

    cbPic->addItem( "Kamerabild" );
    cbPic->addItem( "Bild nach Gaufilter" );
    cbPic->addItem( "Bild nach graukovertierung" );
    cbPic->addItem( "Bild nach Cannyfilter" );
    cbPic->addItem( "Bild nach FindContours" );
    cbPic->setCurrentIndex(activePic);
    connect (cbPic, SIGNAL( currentIndexChanged( int ) ), this, SLOT( setActivePic( int ) ) );

    laySidebar->addWidget(cbPic);

    // Widgets fr den Gaufilter
    grpGaussian = new QGroupBox(tr("Gaufilter"),sidebar);
    layGrpGauss = new QVBoxLayout(grpGaussian);
    layGrpGauss->setMargin(2);

    gaussSizeX = new QSlider(Qt::Horizontal,grpGaussian);
    gaussSizeX->setRange(1,50);
    gaussSizeX->setValue(gaussSzX);
    connect (gaussSizeX, SIGNAL( valueChanged( int ) ), this, SLOT( setGaussSzX( int ) ) );
    layGrpGauss->addWidget(gaussSizeX);

    gaussSizeY = new QSlider(Qt::Horizontal,grpGaussian);
    gaussSizeY->setRange(1,50);
    gaussSizeY->setValue(gaussSzY);
    connect (gaussSizeY, SIGNAL( valueChanged( int ) ), this, SLOT( setGaussSzY( int ) ) );
    layGrpGauss->addWidget(gaussSizeY);

    gaussSigma1 = new QSlider(Qt::Horizontal,grpGaussian);
    gaussSigma1->setRange(0,10000);
    gaussSigma1->setValue(gaussSig1*100);
    connect (gaussSigma1, SIGNAL( valueChanged( int ) ), this, SLOT( setGaussSig1( int ) ) );
    layGrpGauss->addWidget(gaussSigma1);

    gaussSigma2 = new QSlider(Qt::Horizontal,grpGaussian);
    gaussSigma2->setRange(0,10000);
    gaussSigma2->setValue(gaussSig2*100);
    connect (gaussSigma2, SIGNAL( valueChanged( int ) ), this, SLOT( setGaussSig2( int ) ) );
    layGrpGauss->addWidget(gaussSigma2);

    laySidebar->addWidget(grpGaussian);

    // Widgets fr den Cannyfilter
    grpCanny = new QGroupBox(tr("Cannyfilter"),sidebar);
    layGrpCanny = new QVBoxLayout(grpCanny);
    layGrpCanny->setMargin(2);

    cannyTrsh1 = new QSlider(Qt::Horizontal,grpCanny);
    cannyTrsh1->setRange(0,25500);
    cannyTrsh1->setValue(cannyTr1*100);
    connect (cannyTrsh1, SIGNAL( valueChanged( int ) ), this, SLOT( setCannyTr1( int ) ) );
    layGrpCanny->addWidget(cannyTrsh1);

    cannyTrsh2 = new QSlider(Qt::Horizontal,grpCanny);
    cannyTrsh2->setRange(0,25500);
    cannyTrsh2->setValue(cannyTr2*100);
    connect (cannyTrsh2, SIGNAL( valueChanged( int ) ), this, SLOT( setCannyTr2( int ) ) );
    layGrpCanny->addWidget(cannyTrsh2);

    laySidebar->addWidget(grpCanny);

    laySidebar->addStretch();
    layCentral->addWidget(sidebar);

    // Statuszeile des Hauptfensters
    laStatus = new QLabel;
    laStatus->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    laNumContours = new QLabel;
    laNumContours->setFrameStyle(QFrame::Panel | QFrame::Sunken);

    setWindowTitle("Qt ComputerVision");
    setCentralWidget(central);
    statusBar()->addPermanentWidget(laStatus);
    statusBar()->addPermanentWidget(laNumContours);

    startTimer(120);
}

void MainWnd::setGaussSzX( int val ) {
    if (val % 2)
        gaussSzX = val;
    else
        gaussSzX = val + 1;
}

void MainWnd::setGaussSzY( int val ) {
    if (val % 2)
        gaussSzY = val;
    else
        gaussSzY = val + 1;
}

void MainWnd::setGaussSig1( int val ) {
    if (val)
        gaussSig1 = val / 100.0;
    else
        gaussSig1 = 0.0;
}

void MainWnd::setGaussSig2( int val ) {
    if (val)
        gaussSig2 = val / 100.0;
    else
        gaussSig2 = 0.0;
}

void MainWnd::setCannyTr1( int val ) {
    if (val)
        cannyTr1 = val / 100.0;
    else
        cannyTr1 = 0.0;
}

void MainWnd::setCannyTr2( int val ) {
    if (val)
        cannyTr2 = val / 100.0;
    else
        cannyTr2 = 0.0;
}

void MainWnd::setActivePic( int index ) {
    activePic = index;
}

void MainWnd::timerEvent(QTimerEvent*) {
    QTime starttime = QTime::currentTime();
    IplImage dst;

    // Bild von der Kamera laden
    camPic = cvQueryFrame(cam);
    cv::Mat gray, matImage = cv::Mat(camPic);
    if (activePic == 0) {
        dst = *camPic;
        putImage(&dst);
    }

    // Rauschen entfernen
    cv::GaussianBlur(matImage,matImage,cv::Size(gaussSzX,gaussSzY),gaussSig1, gaussSig2);
    if (activePic == 1) {
        dst = matImage;
        putImage(&dst);
    }

    // graustufenbild erzeugen
    cv::cvtColor(matImage,gray,CV_BGR2GRAY);
    if (activePic == 2) {
        dst = gray;
        putImage(&dst);
    }

    // Kantenfilter
    cv::Canny(gray,gray,cannyTr1,cannyTr2);
    if (activePic == 3) {
        dst = gray;
        putImage(&dst);
    }

    std::vector< std::vector < cv::Point > > contours;
    cv::findContours(gray,contours,CV_RETR_CCOMP,CV_CHAIN_APPROX_SIMPLE);
    if (activePic == 4) {
        cv::Scalar color( 255, 255, 255 );
        for (int idx = contours.size() -1; idx >= 0; idx--) {
            cv::Rect bBox = cv::boundingRect(cv::Mat(contours[idx]));
            cv::rectangle(gray,bBox,color);
        }
        dst = gray;
        putImage(&dst);
    }

    qint16 ms = starttime.msecsTo(QTime::currentTime());
    laNumContours->setText(tr("%1").arg(contours.size()));
    laStatus->setText(tr("%1 ms").arg(ms));
}

void MainWnd::putImage(IplImage *cvimage) {
    int cvIndex, cvLineStart;
    // switch between bit depths
    switch (cvimage->depth) {
        case IPL_DEPTH_8U:
            switch (cvimage->nChannels) {
                case 1:
                    if ( (cvimage->width != imPic->width()) || (cvimage->height != imPic->height()) ) {
                        imPic = new QImage(cvimage->width, cvimage->height, QImage::Format_RGB32);
                    }
                    cvIndex = 0; cvLineStart = 0;
                    for (int y = 0; y < cvimage->height; y++) {
                        unsigned char red,green,blue;
                        cvIndex = cvLineStart;
                        for (int x = 0; x < cvimage->width; x++) {
                            // DO it
                            red = cvimage->imageData[cvIndex];
                            green = cvimage->imageData[cvIndex];
                            blue = cvimage->imageData[cvIndex];

                            imPic->setPixel(x,y,qRgb(red, green, blue));
                            cvIndex++;
                        }
                        cvLineStart += cvimage->widthStep;
                    }
                    break;
                case 3:
                    if ( (cvimage->width != imPic->width()) || (cvimage->height != imPic->height()) ) {
                        imPic = new QImage(cvimage->width, cvimage->height, QImage::Format_RGB32);
                    }
                    cvIndex = 0; cvLineStart = 0;
                    for (int y = 0; y < cvimage->height; y++) {
                        unsigned char red,green,blue;
                        cvIndex = cvLineStart;
                        for (int x = 0; x < cvimage->width; x++) {
                            // DO it
                            red = cvimage->imageData[cvIndex+2];
                            green = cvimage->imageData[cvIndex+1];
                            blue = cvimage->imageData[cvIndex+0];

                            imPic->setPixel(x,y,qRgb(red, green, blue));
                            cvIndex += 3;
                        }
                        cvLineStart += cvimage->widthStep;
                    }
                    break;
                default:
                    qDebug() << "This number of channels is not supported\n";
                    break;
            }
            break;
        default:
            qDebug() << "This type of IplImage is not implemented in QOpenCVWidget\n";
            break;
    }
    laPic->setPixmap(QPixmap::fromImage(*imPic));
}

MainWnd::~MainWnd() {
    cvReleaseCapture(&cam);
}
